import postcss from 'postcss';
import * as syntax from 'postcss-less';
import { readFile, existsSync } from 'fs-extra';
const transformPlugin = ({ resolveRoot, atRules, decls, }) => {
    return {
        postcssPlugin: 'transformLessToCssVariablePlugin',
        Root(root) {
            resolveRoot?.(root);
        },
        async AtRule(atRule, helper) {
            if (atRules) {
                Promise.all(atRules.map((fn) => fn(atRule, helper)));
            }
        },
        Declaration(decl, helper) {
            if (decls) {
                Promise.all(decls.map((fn) => fn(decl, helper)));
            }
        },
    };
};
export const transformLessToCssVariable = async (filePath, options) => {
    if (!existsSync(filePath)) {
        throw new Error(`[transformLessToCssVariable] ${filePath} file does not exist.`);
    }
    const fileContent = (await readFile(filePath)).toString('utf-8');
    return postcss([transformPlugin(options)])
        .process(fileContent, { syntax, from: filePath })
        .then((result) => options.formatResult?.(result) ?? result)
        .catch((e) => console.log('postcss error', e));
};
